#pragma once


// COmkListCtrl
#include "OmkListHdrCtrl.h"
#include "OmkListCtrlDataArray.h"

class COmkListCtrl : public CWnd
{
	DECLARE_DYNAMIC(COmkListCtrl)

	enum
	{
		HDR_CTRL_ID     = 0x101,
	};

	enum
	{
		TERM_TEXT   = 4,
		TERM_HEIGHT = 1,
		TERM_WIDTH  = 1,
		HDR_CTRL_HEIGHT = 30,
		ITEM_HEIGHT		= 20,
	};

	enum
	{
		CL_BACK	= RGB(255,255,255),
		CL_TEXT = RGB(0,0,0),
		CL_ITEM_BACK     = RGB(255,190,33),
		CL_BACK_ITEM_SELECTED = RGB(0,0,255),
		CL_TEXT_ITEM_SELECT   = RGB(255,255,255),
	};

	typedef struct LVITEMEX : public LVITEM
	{
		int clText;
		int clBack;
	};

	struct DRAWITEMSTRUCTEX :public DRAWITEMSTRUCT
	{
		int   nColCount;
		UINT  *arrTextAlign;
		COLORREF clText;
		COLORREF clBack;
	};

public:
	COmkListCtrl();
	virtual ~COmkListCtrl();

	BOOL Create(DWORD dwStyle, CWnd *pParent, UINT uID, CRect rect);
	int  InsertColumn(int nCol, 
					  LPCTSTR  lpszColumnHeading, 
					  int nFormat  = LVCFMT_LEFT,
					  int nWidth   = -1,
					  int nSubItem = -1, 
					  int nTextCl  = COmkListHdrCtrl::CL_TEXT, 
					  int nBackCl  = COmkListHdrCtrl::CL_ITEM_BACK);

	int InsertColumn(int nCol,
					 const LVCOLUMN* pColumn, 
					 int nTextCl  = COmkListHdrCtrl::CL_TEXT, 
					 int nBackCl  = COmkListHdrCtrl::CL_ITEM_BACK);

	BOOL GetItem(LVITEMEX *lvItem);
	BOOL GetSubItemRect(int nItem, int nSubItem, CRect &rect);
	BOOL GetItemRect(int nItem, CRect &rect);
	BOOL DeleteColumn(int nCol);
	void DeleteAllColumn();
	
	void SetRedraw(BOOL bRedraw);
	void SetItemHeight(int nHeight, BOOL bRedraw = TRUE);

	int InsertItem(int nItem, LPCTSTR lpszItem, int nImage, int clText = CL_TEXT, int clBack = CL_ITEM_BACK);
	int InsertItem(int nItem, LPCTSTR lpszItem, int clText = CL_TEXT, int clBack = CL_ITEM_BACK); 
	int InsertItem(const LVITEM* pItem, int clText = CL_TEXT, int clBack = CL_ITEM_BACK);

	BOOL SetItemText(int nItem, int nSubItem, LPCTSTR lpszText);
	BOOL DeleteAllItems();
	BOOL DeleteItem(int nItem);

	DWORD   SetExtendedStyle(DWORD dwNewStyle);
	CString GetItemText(int nItem,int nSubItem ) const;

	BOOL GetSelectedItemPos(const CPoint &point, int &nItem, int &nSubItem);

	inline int  GetItemCount(){return (int)m_arrItems.GetSize();}
	inline int  GetSelectionMark() {return m_nSelectItem;}
	inline int  GetTopIndex() {return (abs(m_ptScroll.y) / m_nItemHeight);}
	inline int  GetCountPerPage(){return (m_rectList.Height() / (float)m_nItemHeight)+1.9;}
	inline BOOL SetColumnWidth(int nCol, int cx) {return m_pHdrCtrl.SetItemWidth(nCol, cx);}
	inline DWORD GetExtendedStyle() {return m_dwExtendStyle;} 
	int SetSelectionMark(int iItem);
	BOOL SetAllocationItemCount(int nItemCount, LPCTSTR lpszItem = _T(""), int nImage = 0, int clText = CL_TEXT, int clBack = CL_ITEM_BACK);
	
	BOOL SetCheck(int nItem, BOOL bCheck, BOOL bRedraw = TRUE);
	BOOL GetCheck(int nItem);

private:
	DWORD	 m_dwExtendStyle; 
	int		 m_nSelectItem;

	HFONT	 m_hFont;
	int		 m_nItemHeight;
	COLORREF m_clBack;
	COLORREF m_clBackItemSelected;
	COLORREF m_clTextItemSelected;

	CRect	 m_rectList;

	COmkListCtrlDataArray m_pItemTables;
	CArray <LVITEMEX*>    m_arrItems;
	COmkListHdrCtrl		  m_pHdrCtrl;
	
	CPoint	 m_ptScroll;
	BOOL     m_bRedraw;

protected:
	DECLARE_MESSAGE_MAP()

protected:
	virtual void OnPaint(CDC * pDC, const CRect &rect);
	virtual void DrawItem(CDC * pDC, const DRAWITEMSTRUCTEX *lpDraw);
	
	CRect DrawCheckBox(CDC *pDC, const CRect &rect, const BOOL & bCheck);
	CRect MakeChekBoxRect(const CRect &rect);

	void SetCalcScrollPos(const int nWidth, const int nHeight);
	void ReCalcScrollWidth(SCROLLINFO &info, const int nWidth);

public:
	afx_msg int  OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnPaint();
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnDestroy();
	afx_msg LRESULT OnSetFont(WPARAM wParam, LPARAM lParam);
	afx_msg void OnHdnItemclick(NMHDR *pNMHDR, LRESULT *pResult);
	afx_msg void OnHdnBegintrack(NMHDR *pNMHDR, LRESULT *pResult);

	afx_msg void OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);

	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg BOOL OnMouseWheel(UINT nFlags, short zDelta, CPoint pt);
	afx_msg void OnNMClick(NMHDR *pNMHDR, LRESULT *pResult);
};


