#include "StdAfx.h"
#include "OmkListCtrlDataArray.h"

static inline void ConstructElement(CString* pNewData)
{
	new( pNewData ) CString;
}

static inline void DestructElement(CString* pOldData)
{
	pOldData->~CString();
}

COmkListCtrlDataArray::COmkListCtrlDataArray(void)
{
	m_nColSize = 0;
	m_nRowSize = 0;
	m_nMaxSize = 0;

	m_pData = NULL;
}

COmkListCtrlDataArray::~COmkListCtrlDataArray(void)
{
	RemoveAll();
}

void COmkListCtrlDataArray::RemoveAll()
{
	if(m_pData)
	{
		for (int i =0; i < m_nColSize; i++)
		{
			if(m_pData[i])	
			{
				ColumnDestructArray(m_pData[i], m_nRowSize);
				free(m_pData[i]);
			}
		}

		free(m_pData);
		m_pData = NULL;
	}

	m_nRowSize = 0;
	m_nColSize = 0;
}

int COmkListCtrlDataArray::InsertCol(int nCol)
{
	int nResult = nCol;
	
	if (nCol >= m_nColSize)
	{
		nResult = AddCol() - 1;
	}
	else
	{
		int nMoveCnt = m_nColSize - nCol;

		m_pData	= (CString**)realloc(m_pData, sizeof(CString*) * (m_nColSize+1));
		memmove(&m_pData[nCol+1], &m_pData[nCol], sizeof(CString*) * nMoveCnt);
	
		ReColumnConstructArray(nCol, m_nRowSize);
		
		m_nColSize ++;
	}

	return nResult;
}

int COmkListCtrlDataArray::AddCol()
{
	m_pData	= (CString**)realloc(m_pData, sizeof(CString*) * (m_nColSize+1));

	ReColumnConstructArray(m_nColSize, m_nRowSize);

	m_nColSize ++;
	return m_nColSize;
}

void COmkListCtrlDataArray::ReColumnConstructArray(int nCol, int nRowCount)
{
	m_pData[nCol] = NULL;
	m_pData[nCol] = (CString*)realloc(m_pData[nCol], sizeof(CString)*(nRowCount)); 

	CString *pNewData;
	pNewData  = m_pData[nCol];

	for (int j =0; j < nRowCount; j++)
	{
		ConstructElement(pNewData);
		pNewData ++;
	}
}

void COmkListCtrlDataArray::ColumnDestructArray(CString* pOldData, INT_PTR nCount)
{
	ASSERT(nCount >= 0);

	while (nCount--)
	{
		DestructElement(pOldData);
		pOldData++;
	}
}

void COmkListCtrlDataArray::DeleteCol(int nCol)
{
	ASSERT( !(nCol >= m_nColSize) );

	ColumnDestructArray(m_pData[nCol], m_nRowSize);
	free(m_pData[nCol]);

	int nItemCnt = ((m_nColSize-1) - nCol);
	memmove(&m_pData[nCol], &m_pData[nCol+1], sizeof(CString*) * nItemCnt);

	m_nColSize --;
}

BOOL COmkListCtrlDataArray::SetAllocationRow(int nCount)
{
	ASSERT(!(m_nColSize == 0));

	CString *pNewData;
	for (int i =0; i < m_nColSize; i++)
	{
		// 1. ߰ 迭  Ҵϰ
		m_pData[i] = (CString*)realloc(m_pData[i], sizeof(CString)*(m_nRowSize + nCount)); 
		pNewData   = m_pData[i] + m_nRowSize;

		for (int j =0; j < nCount; j++)
		{
			ConstructElement(pNewData);
			pNewData ++;
		}
	}

	m_nRowSize += nCount;

	return TRUE;
}

int COmkListCtrlDataArray::InsertRow(int nRow)
{
	ASSERT(!(m_nColSize == 0));
	ASSERT(nRow >= 0);    

	int nResult;

	if (nRow >= m_nRowSize)
	{
		nResult = AddRow() - 1;
	}
	else 
	{
		int nMoveCnt = m_nRowSize - nRow;
		
		for (int i =0; i < m_nColSize; i++)
		{
			// 1. ߰ 迭  Ҵϰ
			m_pData[i]  = (CString*)realloc(m_pData[i], sizeof(CString)*(m_nRowSize+1)); 
			
			// 2. 迭 ε Ե ġ ϳ ش.
			memmove(&m_pData[i][nRow+1], &m_pData[i][nRow], nMoveCnt * sizeof(CString));
		
			// 3. Ե  CString  ҴѴ.
			ConstructElement(&m_pData[i][nRow]);
		}
		
		m_nRowSize ++;
		nResult = nRow;
	}

	return nResult;
}

int COmkListCtrlDataArray::AddRow()
{
	ASSERT(!(m_nColSize == 0));
	
	for (int i =0; i < m_nColSize; i++)
	{
		m_pData[i]  = (CString*)realloc(m_pData[i], sizeof(CString)*(m_nRowSize+1)); 
		ConstructElement(&m_pData[i][m_nRowSize]);
	}
	
	m_nRowSize ++;
	return m_nRowSize;
}

CString COmkListCtrlDataArray::GetData(int nCol, int nRow) const 
{
	ASSERT( !(nCol >= m_nColSize||nRow >= m_nRowSize) );
	return m_pData[nCol][nRow];
}

void COmkListCtrlDataArray::SetData(int nCol, int nRow, LPSTR szData)
{
	ASSERT( !(nCol >= m_nColSize||nRow >= m_nRowSize) );

	m_pData[nCol][nRow] = szData;
}

void COmkListCtrlDataArray::DeleteRow(int nRow)
{
	ASSERT( !(nRow >= m_nRowSize) );

	int nItemCnt = ((m_nRowSize-1) - nRow);

	for (int i =0; i < m_nColSize; i++)
	{
		DestructElement(&m_pData[i][nRow]);
		memcpy(&m_pData[i][nRow], &m_pData[i][nRow+1], sizeof(CString) * nItemCnt);
	}

	m_nRowSize --;
}

void COmkListCtrlDataArray::DeleteAllRow()
{
	for (int i =0; i < m_nColSize; i++)
	{
		ColumnDestructArray(m_pData[i], m_nRowSize);
		free(m_pData[i]);
		m_pData[i] = NULL;
	}

	m_nRowSize = 0;
}








